import logging
from abc import abstractmethod
from checkov.common.util.banner import banner


class RunnerRegistry(object):
    runners = []
    scan_reports = []

    def __init__(self, *runners):
        self.logger = logging.getLogger(__name__)
        self.runners = runners

    @abstractmethod
    def extract_entity_details(self, entity):
        raise NotImplementedError()

    def run(self, root_folder, external_checks_dir=None, files=None):
        for runner in self.runners:
            scan_report = runner.run(root_folder, external_checks_dir=external_checks_dir, files=files)
            self.scan_reports.append(scan_report)
        return self.scan_reports

    @staticmethod
    def print_reports(scan_reports, args):
        if args.output != "json" and args.output != "junitxml":
            print(f"{banner}\n")
        exit_codes = []
        for report in scan_reports:
            if not report.is_empty():
                if args.output == "json":
                    report.print_json()
                elif args.output == "junitxml":
                    report.print_junit_xml()
                else:
                    report.print_console()
            exit_codes.append(report.get_exit_code(args.soft_fail))
        exit_code = 1 if 1 in exit_codes else 0
        exit(exit_code)



from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck

PORT = '22'


class GoogleComputeFirewallUnrestrictedIngress22(BaseResourceCheck):
    def __init__(self):
        name = "Ensure Google compute firewall ingress does not allow unrestricted ssh access"
        id = "CKV_GCP_2"
        supported_resources = ['google_compute_firewall']
        categories = [CheckCategories.NETWORKING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        """
            Looks for password configuration at google_compute_firewall:
            https://www.terraform.io/docs/providers/google/r/compute_firewall.html
        :param conf: azure_instance configuration
        :return: <CheckResult>
        """
        allow_blocks = conf['allow']
        for block in allow_blocks:
            if 'ports' in block.keys():
                if PORT in block['ports'][0]:
                    if 'source_ranges' in conf.keys():
                        source_ranges = conf['source_ranges'][0]
                        if "0.0.0.0/0" in source_ranges:
                            return CheckResult.FAILED
        return CheckResult.PASSED


check = GoogleComputeFirewallUnrestrictedIngress22()



from checkov.terraform.context_parsers.base_parser import BaseContextParser
import os
import dpath.util


class VariableContextParser(BaseContextParser):
    def __init__(self):
        definition_type = 'variable'
        super().__init__(definition_type=definition_type)

    def get_block_type(self):
        return self.definition_type

    def _collect_default_variables_values(self, variable_block):
        (variable_folder, _) = os.path.split(self.tf_file)
        for variable_name, values in variable_block.items():
            if 'default' in values.keys():
                for key, value in values.items():
                    if isinstance(value, list) and len(value) == 1:
                        value = values['default'][0]
                        if type(value) in (int, float, bool, str):
                            dpath.new(self.context, "//".join(['assignments', variable_name]), value,
                                      separator='//')

    def enrich_definition_block(self, block):
        # self.context = super().enrich_definition_block(block) # TODO, the base class does not generalize to a variable block
        self.context = {}
        for i, variable_block in enumerate(block):
            self._collect_default_variables_values(variable_block)
        return self.context


parser = VariableContextParser()



from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class CloudfrontDistributionEncryption(BaseResourceCheck):
    def __init__(self):
        name = "Ensure cloudfront distribution ViewerProtocolPolicy is set to HTTPS"
        id = "CKV_AWS_34"
        supported_resources = ['aws_cloudfront_distribution']
        categories = [CheckCategories.ENCRYPTION]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        """
            Looks for ViewerProtocolPolicy configuration at cloudfront distributions:
            https://www.terraform.io/docs/providers/aws/r/cloudfront_distribution.html#viewer_protocol_policy
        :param conf: cloudfront configuration
        :return: <CheckResult>
        """
        if "default_cache_behavior" in conf.keys():
            default_viewer_policy = conf["default_cache_behavior"][0]["viewer_protocol_policy"][0]
            if default_viewer_policy == "allow-all":
                return CheckResult.FAILED
        if "ordered_cache_behavior" in conf.keys():
            for behavior in conf["ordered_cache_behavior"]:
                if behavior["viewer_protocol_policy"][0] == "allow-all":
                    return CheckResult.FAILED
        return CheckResult.PASSED


check = CloudfrontDistributionEncryption()



from checkov.terraform.context_parsers.base_parser import BaseContextParser


class ProviderContextParser(BaseContextParser):
    def __init__(self):
        definition_type = 'provider'
        super().__init__(definition_type=definition_type)

    def get_block_type(self):
        return self.definition_type

    def get_entity_name_and_type(self, entity_block):
        entity_type = self.definition_type
        entity_name = next(iter(entity_block))
        return entity_name, entity_type


parser = ProviderContextParser()



import re

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.provider.base_check import BaseProviderCheck


class AWSCredentials(BaseProviderCheck):
    access_key_pattern = "(?<![A-Z0-9])[A-Z0-9]{20}(?![A-Z0-9])"
    secret_key_pattern = "(?<![A-Za-z0-9/+=])[A-Za-z0-9/+=]{40}(?![A-Za-z0-9/+=])"

    def __init__(self):
        name = "Ensure no hard coded AWS access key and and secret key exists"
        id = "CKV_AWS_41"
        supported_provider = ['aws']
        categories = [CheckCategories.GENERAL_SECURITY]
        super().__init__(name=name, id=id, categories=categories, supported_provider=supported_provider)

    def scan_provider_conf(self, conf):
        """
        see: https://www.terraform.io/docs/providers/aws/index.html#static-credentials
        """
        if self.secret_found(conf, "access_key", self.access_key_pattern):
            return CheckResult.FAILED
        if self.secret_found(conf, "secret_key", self.secret_key_pattern):
            return CheckResult.FAILED
        return CheckResult.PASSED

    @staticmethod
    def secret_found(conf, field, pattern):
        if field in conf.keys():
            value = conf[field][0]
            if re.match(pattern, value) is not None:
                return True
        return False


check = AWSCredentials()



from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class IAMPolicyAttachedToGroupOrRoles(BaseResourceCheck):
    def __init__(self):
        name = "Ensure IAM policies are attached only to groups or roles (Reducing access management complexity may " \
               "in-turn reduce opportunity for a principal to inadvertently receive or retain excessive privileges.) "
        id = "CKV_AWS_40"
        supported_resources = ['aws_iam_user_policy_attachment', 'aws_iam_user_policy', 'aws_iam_policy_attachment']
        categories = [CheckCategories.IAM]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        if 'user' in conf.keys() or 'users' in conf.keys():
            return CheckResult.FAILED
        return CheckResult.PASSED

check = IAMPolicyAttachedToGroupOrRoles()
